<?php

declare(strict_types=1);

namespace Drupal\tmgmt_workspaces\EventSubscriber;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\SynchronizableInterface;
use Drupal\Core\Entity\TranslatableInterface;
use Drupal\workspaces\Event\WorkspacePrePublishEvent;
use Drupal\workspaces\WorkspaceManagerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Don't remove translations added to live that aren't in workspace.
 */
final readonly class TmgmtWorkspacesSubscriber implements EventSubscriberInterface {

  public function __construct(
    private WorkspaceManagerInterface $workspaceManager,
    private EntityTypeManagerInterface $entityTypeManager,
  ) {}

  public function onWorkspacePrePublish(WorkspacePrePublishEvent $event): void {
    if ($event->isPublishingStopped()) {
      return;
    }
    $revisionIds = $event->getPublishedRevisionIds();
    $workspaceId = $event->getWorkspace()->id();
    \assert(\is_string($workspaceId));
    foreach ($revisionIds as $entityTypeId => $revisionIdList) {
      $storage = $this->entityTypeManager->getStorage($entityTypeId);
      foreach ($revisionIdList as $revisionId) {
        $workspaceEntity = $this->workspaceManager->executeInWorkspace($workspaceId, static fn() => $storage->load($revisionId));
        if ($workspaceEntity === NULL) {
          continue;
        }
        if (!$workspaceEntity instanceof TranslatableInterface) {
          continue;
        }
        $this->workspaceManager->executeOutsideWorkspace(function () use ($workspaceEntity, $storage, $workspaceId): void {
          $liveRevision = $storage->loadUnchanged($workspaceEntity->id());
          \assert($liveRevision instanceof TranslatableInterface);
          $updateTranslation = FALSE;
          foreach ($liveRevision->getTranslationLanguages() as $language) {
            if (!$workspaceEntity->hasTranslation($language->getId())) {
              $workspaceEntity->addTranslation($language->getId(), $liveRevision->getTranslation($language->getId())->toArray());
              $updateTranslation = TRUE;
            }
          }
          if ($updateTranslation) {
            $this->workspaceManager->executeInWorkspace($workspaceId, static function () use ($workspaceEntity): void {
              if ($workspaceEntity instanceof SynchronizableInterface) {
                $workspaceEntity->setSyncing(TRUE);
              }
              $workspaceEntity->save();
            });
          }
        });
      }
    }

  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      WorkspacePrePublishEvent::class => ['onWorkspacePrePublish'],
    ];
  }

}
