<?php

declare(strict_types=1);

namespace Drupal\tmgmt_workspaces;

use Drupal\tmgmt\Entity\Job;
use Drupal\workspaces\WorkspaceInterface;

/**
 * Entity class override for the tmgmt_job entity.
 */
class WorkspaceAwareJob extends Job {

  /**
   * {@inheritdoc}
   */
  public function getConflictingItemIds() {
    // @todo Show the conflicting items in the Job form as well.
    // @see \Drupal\tmgmt\Form\JobForm::getConflictingItemsMessage()
    return array_unique(array_merge(parent::getConflictingItemIds(), $this->getWorkspaceConflictingItemsIds()));
  }

  /**
   * Returns workspace-conflicting job item IDs.
   *
   * Workspace-conflicting job items are those that have the source entity
   * already tracked in a different workspace.
   *
   * @return int[]
   *   List of conflicting job item IDs.
   */
  public function getWorkspaceConflictingItemsIds(): array {
    // Return early if there's no target workspace configured.
    if ($this->get('target_workspace')->isEmpty()) {
      return [];
    }

    /** @var \Drupal\workspaces\WorkspaceAssociationInterface $workspace_association */
    $workspace_association = \Drupal::service('workspaces.association');
    $target_workspace = $this->get('target_workspace')->target_id;

    $workspace_conflicting_item_ids = [];
    foreach ($this->getItems() as $item) {
      if ($item->getPlugin() !== 'content') {
        continue;
      }

      /** @var \Drupal\Core\Entity\RevisionableInterface $entity */
      $entity = $this->entityTypeManager()->getStorage($item->getItemType())->load($item->getItemId());
      if ($tracking_workspace_ids = $workspace_association->getEntityTrackingWorkspaceIds($entity)) {
        if (!in_array($target_workspace, $tracking_workspace_ids, TRUE)) {
          $workspace_conflicting_item_ids[] = $item->id();
        }
      }
    }

    return $workspace_conflicting_item_ids;
  }

  /**
   * Returns entity IDs contained in workspace-conflicting job items.
   *
   * As long as the job is not saved, job items don't have an id, so conflicts
   * need to be computed based on the entities they reference. This is required
   * to determined conflicting suggestions.
   *
   * @return int[]
   *   List of conflicting entity IDs.
   *
   * @todo Check if this code can be combined with the above method somehow.
   */
  public static function getWorkspaceConflictingEntityIds(?string $target_workspace_id, array $job_items): array {
    /** @var \Drupal\workspaces\WorkspaceAssociationInterface $workspace_association */
    $workspace_association = \Drupal::service('workspaces.association');
    $entity_type_manager = \Drupal::entityTypeManager();

    $workspace_conflicting_entity_ids = [];
    foreach ($job_items as $item) {
      if ($item->getPlugin() !== 'content') {
        continue;
      }

      /** @var \Drupal\Core\Entity\RevisionableInterface $entity */
      $entity = $entity_type_manager->getStorage($item->getItemType())->load($item->getItemId());
      if ($tracking_workspace_ids = $workspace_association->getEntityTrackingWorkspaceIds($entity)) {
        // If the target workspace is live and the entity referenced by the job
        // item is edited in any workspace OR the target worksüace is not live
        // and the entity is edited in a workspace any other than the current
        // one or its descendants, there is a conflict.
        if ((\is_null($target_workspace_id) && $tracking_workspace_ids !== []) || !in_array($target_workspace_id, $tracking_workspace_ids, TRUE)) {
          $message = \Drupal::translation()->formatPlural(
            \count($tracking_workspace_ids),
            '<strong>This @entity_type_label has conflicts with the following workspace and cannot be added to the job</strong>:&nbsp;',
            '<strong>This @entity_type_label has conflicts with the following other workspaces and cannot be added to the job.</strong>:&nbsp;',
            ['@entity_type_label' => $entity->getEntityType()->getSingularLabel()]
          );
          $tracking_workspaces = $entity_type_manager->getStorage('workspace')
            ->loadMultiple($tracking_workspace_ids);
          $workspaces_links = \array_map(
            static fn (WorkspaceInterface $workspace) => '<a href="' . $workspace->toUrl()->toString() . '">' . $workspace->label() . '</a>',
            $tracking_workspaces
          );
          $message .= implode(', ', $workspaces_links);
          $workspace_conflicting_entity_ids[$entity->id()] = $message;
        }
      }
    }

    return $workspace_conflicting_entity_ids;
  }


}
